#region Copyright (C) 2006 Torq Software.
//=============================================================================
// Copyright 2006 Torq Software - http://www.torqsoftware.com
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//=============================================================================
#endregion Copyright (C) 2006 Torq Software.

#if INCLUDE_TESTS

using System;
using System.ComponentModel;

namespace Torq.Foundation.Tests
{
    /// <summary>
    /// Generates events for the <see cref="WeakDelegateTest"/> class.
    /// </summary>
    internal class TestEventGenerator
    {
        private Delegator statusChanged;
        private Delegator listChanged;
        private int statusChangedEventCount;
        private int listChangedEventCount;

        /// <summary>
        /// Occurs when the status of this test class has changed.
        /// </summary>
        public event EventHandler StatusChanged
        {
            add { Delegator.Add( ref statusChanged, value ); }
            remove { Delegator.Remove( ref statusChanged, value ); }
        }

        /// <summary>
        /// An event whose add/remove methods are the standard ones defined by
        /// the compiler.
        /// </summary>
        public event EventHandler StandardStatusChange;

        /// <summary>
        /// Raises the standard status changed event.
        /// </summary>
        public void RaiseStandardStatusChanged()
        {
            if ( StandardStatusChange != null )
                StandardStatusChange( this, EventArgs.Empty );
        }

        /// <summary>
        /// Gets the status changed event count.
        /// </summary>
        /// <value>The status changed event count.</value>
        public int StatusChangedEventCount
        {
            get { return statusChangedEventCount; }
        }

        /// <summary>
        /// Gets the list changed event count.
        /// </summary>
        /// <value>The list changed event count.</value>
        public int ListChangedEventCount
        {
            get { return listChangedEventCount; }
        }

        /// <summary>
        /// Raise the status changed event.
        /// </summary>
        public void RaiseStatusChanged()
        {
            ++statusChangedEventCount;
            Delegator.InvokeHandler( ref statusChanged, this, EventArgs.Empty );
        }
        
        /// <summary>
        /// Occurs when the status of this test class has changed.
        /// </summary>
        public event ListChangedEventHandler ListChanged
        {
            add { Delegator.Add( ref listChanged, value ); }
            remove { Delegator.Remove( ref listChanged, value ); }
        }

        /// <summary>
        /// Raise the list changed event.
        /// </summary>
        /// <param name="changeType">The type of change.</param>
        /// <param name="newIndex">The new index.</param>
        /// <param name="oldIndex">The old index.</param>
        public void RaiseListChanged( ListChangedType changeType, int newIndex, int oldIndex )
        {
            ++listChangedEventCount;
            ListChangedEventArgs args = new ListChangedEventArgs( changeType, newIndex, oldIndex );
            Delegator.InvokeListChangedHandler( ref listChanged, this, args );
        }        
    }
}

#endif